--[[--------------------------------------------------------------------------------
  ItemSyncCore Core Framework

  Author:  Derkyle
  Website: http://www.manaflux.com
-----------------------------------------------------------------------------------]]

BINDING_HEADER_ITEMSYNC_HEADER = "ItemSync";


--hooks
local ISync_lOriginal_ChatFrame_OnEvent;
local ISync_lOriginal_MerchantFrame_Update;
local ISync_lOriginal_ChatEdit_OnTextChanged;


--Hook chat messages
local ISync_ChatMessageTypes = {
	"CHAT_MSG_SYSTEM",
	"CHAT_MSG_SAY",
	"CHAT_MSG_TEXT_EMOTE",
	"CHAT_MSG_YELL",
	"CHAT_MSG_WHISPER",
	"CHAT_MSG_PARTY",
	"CHAT_MSG_GUILD",
	"CHAT_MSG_OFFICER",
	"CHAT_MSG_CHANNEL",
	"CHAT_MSG_RAID",
	"CHAT_MSG",
	"CHAT_MSG_LOOT",
};
--Hook inventory slots
ISyncCore_InvList = {
	{ name = "HeadSlot" },
	{ name = "NeckSlot" },
	{ name = "ShoulderSlot" },
	{ name = "BackSlot" },
	{ name = "ChestSlot" },
	{ name = "ShirtSlot" },
	{ name = "TabardSlot" },
	{ name = "WristSlot" },
	{ name = "HandsSlot" },
	{ name = "WaistSlot" },
	{ name = "LegsSlot" },
	{ name = "FeetSlot" },
	{ name = "Finger0Slot" },
	{ name = "Finger1Slot" },
	{ name = "Trinket0Slot" },
	{ name = "Trinket1Slot" },
	{ name = "MainHandSlot" },
	{ name = "SecondaryHandSlot" },
	{ name = "RangedSlot" },
};



---------------------------------------------------
-- ISyncCore_OnLoad
---------------------------------------------------
function ISyncCore_OnLoad()


	this:RegisterEvent("VARIABLES_LOADED");
	this:RegisterEvent("PLAYER_ENTERING_WORLD");
	this:RegisterEvent("PLAYER_LEAVING_WORLD");
	this:RegisterEvent("PLAYER_TARGET_CHANGED");
	--this:RegisterEvent("UNIT_INVENTORY_CHANGED"); -- was removed to streamline loading
	this:RegisterEvent("BANKFRAME_OPENED");
	this:RegisterEvent("AUCTION_ITEM_LIST_UPDATE");
	this:RegisterEvent("MERCHANT_SHOW");
	this:RegisterEvent("BAG_UPDATE");
	this:RegisterEvent("ADDON_LOADED");


	--register chat types
	for index, value in ISync_ChatMessageTypes do
		this:RegisterEvent(value);
	end

	--Hook the ChatFrame_OnEvent function
	ISync_lOriginal_ChatFrame_OnEvent = ChatFrame_OnEvent;
	ChatFrame_OnEvent = ISyncCore_ChatFrame_OnEvent;
	
	--Hook ChatEdit_OnTextChanged for type links
	ISync_lOriginal_ChatEdit_OnTextChanged = ChatEdit_OnTextChanged;
	ChatEdit_OnTextChanged = ISyncCore_ChatEdit_OnTextChanged;
	
	
	--Hook for the Merchant Prices
	ISync_lOriginal_MerchantFrame_Update = MerchantFrame_Update;
	MerchantFrame_Update = ISyncCore_MerchantFrame_Update;
	
	
	--pass to main framework
	if(ISync) then ISync:OnLoad(); end


end


---------------------------------------------------
-- ISyncCore_OnEvent
---------------------------------------------------
function ISyncCore_OnEvent()

	--hook onto the armor slots and weapon slots
	if( event == "VARIABLES_LOADED" ) then

		--register the ivnentory slots
		for index = 1, getn(ISyncCore_InvList), 1 do
			ISyncCore_InvList[index].id = GetInventorySlotInfo(ISyncCore_InvList[index].name);
		end

	end
	
	--special thanks to Silowyi for pointing this out.
	if( event == "PLAYER_LEAVING_WORLD" ) then
		ISync_CoreFrame:UnregisterEvent("UNIT_INVENTORY_CHANGED");
		ISync_CoreFrame:UnregisterEvent("BAG_UPDATE");
	end
	if( event == "PLAYER_ENTERING_WORLD" ) then
		ISync_CoreFrame:RegisterEvent("UNIT_INVENTORY_CHANGED");
		ISync_CoreFrame:RegisterEvent("BAG_UPDATE");
	end
	
	

	--pass to main framework
	if(ISync) then ISync:OnEvent(event,arg1,arg2,arg3,arg4,arg5); end
	if(ISyncTooltip_OnEvent) then ISyncTooltip_OnEvent(event,arg1,arg2,arg3,arg4,arg5); end

end


---------------------------------------------------
-- ISyncCore_OnUpdate
---------------------------------------------------
function ISyncCore_OnUpdate(elapsed)

	--pass to main framework
	if(ISync) then ISync:OnUpdateTriggered(elapsed); end

end


---------------------------------------------------
-- ISyncCore_ChatFrame_OnEvent
---------------------------------------------------
function ISyncCore_ChatFrame_OnEvent(event)

	
	--pass to main framework
	if(ISync) then
		
		--if there return nil then pass onto original chatframe
		if (not ISync:OnChatParse(event,arg1,arg2,arg3,arg4,arg5)) then
		
			ISync_lOriginal_ChatFrame_OnEvent(event);
			
		end
	
	end
	
	

end


---------------------------------------------------
-- ISyncCore_MerchantFrame_Update
---------------------------------------------------
function ISyncCore_MerchantFrame_Update()
	ISync_lOriginal_MerchantFrame_Update(); --do original
	
	if(ISync) then ISync:VendorScan(); end
end


--------------------------------------------------------------------------------------------------------------------------------
--------------------------------------------------------------------------------------------------------------------------------
--------------------------------------------------------------------------------------------------------------------------------
--------------------------------------------------------------------------------------------------------------------------------
--------------------------------------------------------------------------------------------------------------------------------
--------------------------------------------------------------------------------------------------------------------------------

--TYPELINKS FUNCTIONS
--Special thanks to Gazmik Fizzwidget for the original code

---------------------------------------------------
-- ISyncCore_ChatEdit_OnTextChanged
---------------------------------------------------
function ISyncCore_ChatEdit_OnTextChanged()
	 
	--check for options
	if( not ISyncOpt ) then ISyncOpt = { }; end
	if( not ISyncOpt.GENERAL_EnableTypeLinks) then ISyncOpt.GENERAL_EnableTypeLinks = 0; end
	
	if(ISyncOpt.GENERAL_EnableTypeLinks == 1) then
	
		local text = this:GetText();
		text = ISyncCore_ParseChatMSG(text);    
		this:SetText(text);
		ISync_lOriginal_ChatEdit_OnTextChanged(this);
	
	else
		ISync_lOriginal_ChatEdit_OnTextChanged(this);
	end

end


---------------------------------------------------
-- ISyncCore_ParseChatMSG
---------------------------------------------------
function ISyncCore_ParseChatMSG(text)
	return string.gsub(text, "([|]?[h]?)%[(.-)%]([|]?[h]?)", ISyncCore_SetTextLink);
end



---------------------------------------------------
-- ISyncCore_SetTextLink
---------------------------------------------------
function ISyncCore_SetTextLink(head, text, tail)
	if (head ~= "|h" and tail ~= "|h") then
		local link = ISyncCore_GetLinkByName(text);
		if (link) then return link; end
	end
	return head.."["..text.."]"..tail;
end




---------------------------------------------------
-- ISyncCore_GetLinkByName
---------------------------------------------------
function ISyncCore_GetLinkByName(text, returnAll)
	local _, _, name, property = string.find(text, "(.+)%((.-)%)" );
	local sChk = 0;
	local sChkInfo;
	local sChkName;
	
	if( not ISyncDB ) then return nil; end
	if( not ISync_RealmNum) then return nil; end
	if( not ISyncDB[ISync_RealmNum]) then return nil; end
	if( not ISync) then return nil; end

	for index, value in ISyncDB[ISync_RealmNum] do

		if(string.lower(index) == string.lower(text)) then
			sChk = 1;
			sChkInfo = value;
			sChkName = index;
		end

	end
	
	if(sChk == 0 and not tonumber(text)) then return nil; end
	
	
	if(sChk == 1 and sChkInfo) then

		local s1, s2, sParseInfo = string.find(sChkInfo, "(.-)");
		
		if(sParseInfo) then
	
			--check item before we add it
			local name_X, link_X, quality_X, minLevel_X, class_X, subclass_X, maxStack_X = GetItemInfo("item:"..sParseInfo);

			--check for info and stuff
			if(link_X and quality_X) then

				--grab the color using the CheckColor routine
				useColor = ISync:CheckColor(nil, tonumber(quality_X), 2)
				
				return "|c"..useColor.."|H"..link_X.."|h["..sChkName.."]|h|r";

			end
		
		end
				
			
	--they are looking for a number
	else
	
		return nil;
	
	end
	

end


--------------------------------------------------------------------------------------------------------------------------------
--------------------------------------------------------------------------------------------------------------------------------
--------------------------------------------------------------------------------------------------------------------------------
--------------------------------------------------------------------------------------------------------------------------------
--------------------------------------------------------------------------------------------------------------------------------
--------------------------------------------------------------------------------------------------------------------------------


---------------------------------------------------
-- ISyncCore_FixPattern
---------------------------------------------------
function ISyncCore_FixPattern(sText)
local result = "";
local remain = sText;
local char;

local ISyncCore_Chars = { };
ISyncCore_Chars["^"] = 1;
ISyncCore_Chars["$"] = 1;
ISyncCore_Chars["("] = 1;
ISyncCore_Chars[")"] = 1;
ISyncCore_Chars["%"] = 1;
ISyncCore_Chars["."] = 1;
ISyncCore_Chars["["] = 1;
ISyncCore_Chars["]"] = 1;
ISyncCore_Chars["*"] = 1;
ISyncCore_Chars["+"] = 1;
ISyncCore_Chars["-"] = 1;
ISyncCore_Chars["?"] = 1;

	while( remain ~= "" ) do
		char = strsub(remain, 1, 1);
		if( ISyncCore_Chars[char] ) then
			result = result.."%"..char;
		else
			result = result..char;
		end
		remain = strsub(remain, 2);
	end
	
	return result;
end


---------------------------------------------------
-- ISyncCore_MiniMapMouseCheck
---------------------------------------------------
function ISyncCore_MiniMapMouseCheck()
   local result = 0;
   
   --special thanks to the creator of TrackerAssist
   if ((MinimapCluster and MouseIsOver(MinimapCluster) and MinimapCluster:IsVisible() and
        MiniMapTrackingFrame and (not MouseIsOver(MiniMapTrackingFrame)) and
        MiniMapMailFrame and (not MouseIsOver(MiniMapMailFrame)) and
        MinimapZoomIn and (not MouseIsOver(MinimapZoomIn)) and
        MinimapZoneTextButton and (not MouseIsOver(MinimapZoneTextButton)) and
        MinimapZoomOut and (not MouseIsOver(MinimapZoomOut)) and
        GameTimeFrame and (not MouseIsOver(GameTimeFrame))) or
       (ZoomMapCluster and MouseIsOver(ZoomMapCluster) and ZoomMapCluster:IsVisible())) then
       result = 1;
   end
   
   return result;
end


---------------------------------------------------
-- ISyncCore_GetRealm
---------------------------------------------------
function ISyncCore_GetRealm()
	--for some reason I've realized that some names that are returned
	--from the realm listing is wrong.  Therefore we have to trim the
	--information so we can use it properly.
	
	if(not GetCVar("realmName")) then return nil; end
	
	local chkString = string.gsub(GetCVar("realmName"), "^%s*(.-)%s*$", "%1");
	
	if(chkString) then return chkString; else return nil; end
	
end

---------------------------------------------------
-- ISyncCore_AddRealm
---------------------------------------------------
function ISyncCore_AddRealm()
		
	--check for options
	if( not ISyncOpt ) then ISyncOpt = { }; end
	if( not ISyncOpt.Realms) then ISyncOpt.Realms = { }; end
	if( not ISyncOpt.RealmCount) then ISyncOpt.RealmCount = 0; end
	if( not ISyncOpt.SERVER_MergeDB) then ISyncOpt.SERVER_MergeDB = 0; end
	
	--get the realm
	if(not ISyncCore_GetRealm()) then return nil; end
	
	--add it
	if(not ISyncOpt.Realms[ISyncCore_GetRealm()]) then
		ISyncOpt.Realms[ISyncCore_GetRealm()] = ISyncOpt.RealmCount;
		ISyncOpt.RealmCount = ISyncOpt.RealmCount + 1;
		
		--if they want to merge the database then always send zero
		if(ISyncOpt.SERVER_MergeDB == 1) then 
			return 0; 
		else
			return ISyncOpt.Realms[ISyncCore_GetRealm()];
		end
	
	else

		--if they want to merge the database then always send zero
		if(ISyncOpt.SERVER_MergeDB == 1) then 
			return 0; 
		else
			return ISyncOpt.Realms[ISyncCore_GetRealm()];
		end
		
	end
	
end